using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// HZMBathroomTiny v1.0
//
// Allows modifications of the bathroom tiny asset

namespace HZMBathroomTinyPlugin
{
    public class HZMBathroomTiny : MVRScript
    {
		
		public JSONStorableStringChooser DoorType;
		public JSONStorableBool EnableShowerLeds;
		public JSONStorableBool EnableMirrorLed;
		public JSONStorableBool EnableHideWalls;
		protected List<string> DoorTypeChoices;
			
		public Transform ObjectRoot;
		public Transform DoorsRoot;
		public Transform LedsRoot;
		public Transform MirrorLedRoot;
		public Transform TransparencyRoot;

		private JSONStorableString _loadingInfos;
		private JSONStorableString _helpText;

		private bool seInitialized = false;
		
		// Security and initialization
		private bool editorInitialized = false;
		private int initLoops = 0;
		private int maxInitLoops = 500;
		private bool cuaIsValid = false;
		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add HZMBathroomTiny on a compatible Custom Unity Asset.");
                    return;
                }
				
				// Type of door
				DoorTypeChoices = new List<string>();
				DoorTypeChoices.Add("Door static");
				DoorTypeChoices.Add("Door with physic");
				
				DoorType = new JSONStorableStringChooser("Door Type", DoorTypeChoices, "Door static", "Door Type", DoorTypeCallback);

				_helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help - HZM Bathroom Tiny</b></size></color>\n\n" + 
					"<color=#333>" +
					"This plugin allows you to customize several components of the bathroom tiny.\n\n" +
					"<b>Enable walls transparency :</b> is an option that is <b>not</b> saved with the scene. It allows you to work without the walls and create your scene without having the walls obscuring the view.\n\nIt keeps the collisions active (to avoid issue with characters using the walls as a support for instance).\n\n" +
					"</color>"
				);

				EnableShowerLeds = new JSONStorableBool("Enable Shower LEDs", true, EnableShowerLedsCallback);
				EnableMirrorLed = new JSONStorableBool("Enable Mirror LED", true, EnableMirrorLedCallback);
				EnableHideWalls = new JSONStorableBool("Enable walls transparency", false, EnableHideWallsCallback);

								
				RegisterStringChooser(DoorType);
				RegisterBool(EnableShowerLeds);
				RegisterBool(EnableMirrorLed);

				// Creating the default UI that will inform you that the thing is loading
				CreateLoadingUI();
				
				// ************************************************
				// ** ACTIONS
				// ************************************************			


			}
            catch(Exception e)
            {
                SuperController.LogError("HZMBathroomTiny - Exception caught: " + e);
            }
		}
		
		void Start()
        {
			
        }
		
		// Used for the initialization of the editor.
		// When it is initialized, there's nothing done every frame
		void Update()
        {
			// If the editor is not initialized, we're gonna try to do it
			// I'm putting 3600 tries ( at 60fps it is 1min )
			if( initLoops < maxInitLoops && editorInitialized == false ) {
				initLoops++;
				// If our CUA is loading and everything is ok, we can remove the loading UI and Init the final UI
				if (InitEditor() == true)
				{
					RemoveLoadingUI();
					cuaIsValid = true; // Our CUA is initialized, we consider it as valid
					StartCoroutine("CheckCUAStatus"); // We're gonna check if the CUA is valid every 5 secs
				}
			}
			
			// Triggering an error if we have reached our limit of tries
			if( initLoops == maxInitLoops && editorInitialized == false ) {
				initLoops++; // To avoid spamming the log with the error
			}
		}
		
		void OnDisable() {

		}
		
		private bool InitEditor() {
			// Updating our loading UI
			UpdateLoadingUI();

			if( seInitialized == true ) return true;
			
			ObjectRoot = getObjectRoot( containingAtom.reParentObject, "bathroom_tiny_enviro" );
			if( ObjectRoot == null ) {
				return false;
			}
			
			DoorsRoot = getChildRoot( ObjectRoot, "shower_doors" );
			if( DoorsRoot == null ) {
				return false;
			}
			
			LedsRoot = getChildRoot( ObjectRoot, "shower_leds" );
			if( LedsRoot == null ) {
				return false;
			}
			
			MirrorLedRoot = getChildRoot( ObjectRoot, "mirror_led" );
			if( MirrorLedRoot == null ) {
				return false;
			}
			
			TransparencyRoot = getChildRoot( ObjectRoot, "transparency_parent" );
			if( TransparencyRoot == null ) {
				return false;
			}
			
			ChangeDoorType( DoorType.val );
			EnableShowerLedsCallback( EnableShowerLeds.val );
			EnableMirrorLedCallback( EnableMirrorLed.val );

			seInitialized = true;
			CreateCUAUI();
			
			return true;

		}
		
		// The function creating the loading UI before everything is ready
		private void CreateLoadingUI()
		{
			_loadingInfos = new JSONStorableString("Loading Infos", "");
			UIDynamic stateinfosTextfield = CreateTextField(_loadingInfos, false);
			stateinfosTextfield.height = 450.0f;
		}

		// The function clearing the loading UI
		private void RemoveLoadingUI()
		{
			// Removing the loading text field on the left of the UI
			RemoveTextField(_loadingInfos);
		}

		// The function updating the loading UI while we're either waiting for the CUA to load, or waiting for the root object and everything to be found
		private void UpdateLoadingUI()
		{
			var loadingText = "<color=#000><b>Custom Unity Asset loading...</b></color>\n" + (initLoops/60) + "/" + (maxInitLoops/60) + "secs";
			if (initLoops == maxInitLoops)
			{
				loadingText += "\n\n<color=#333><b>UNABLE TO LOAD YOUR CUA :</b>\n";
				loadingText += "- Your CUA may not be compatible with this script\n";
				loadingText += "- Your CUA may take too long to load</color>";
				loadingText += "\n\nSelect another CUA and reload the script, or just reload the script to try again.";
			}
			_loadingInfos.val = loadingText;
		}
		
		// The function creating the UI when the CUA has been invalidated
		private void CreateCUAInvalidatedUI()
		{
			JSONStorableString _invalidatedCUA = new JSONStorableString("Invalidated CUA", "");
			UIDynamic stateinfosTextfield = CreateTextField(_invalidatedCUA, false);
			stateinfosTextfield.height = 150.0f;
			_invalidatedCUA.val = "<color=#000><b>Custom Unity Asset invalidated.</b></color>\n" +
			                      "You have changed the CUA in the <b>Asset</b> tab, the script is no longer valid. Please reload it or change it.";
		}

		// Function creating the whole CUA UI
		private void CreateCUAUI()
		{
			UIDynamicPopup DTdp = CreatePopup(DoorType, false);
			DTdp.labelWidth = 200f;
			
			CreateToggle(EnableShowerLeds, false);
			CreateToggle(EnableMirrorLed, false);
			CreateToggle(EnableHideWalls, false);
			
			UIDynamic helpTextfield = CreateTextField(_helpText, true);
			helpTextfield.height = 600.0f;
		}
		
		// Function clearing the whole CUA UI
		private void ClearCUAUI()
		{
			RemovePopup(DoorType);
			RemoveToggle(EnableShowerLeds);
			RemoveToggle(EnableMirrorLed);
			RemoveToggle(EnableHideWalls);
			RemoveTextField(_helpText);
		}
		
		// Recursive function searching for a transform containing the string set in objectNameSearchString
		// It is used the first time for the initialization of the script
		// This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
		private Transform getObjectRoot( Transform parent, string rootName ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(rootName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child,rootName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		private Transform getChildRoot( Transform parent, string searchName ) {
			foreach( Transform child in parent )
			{
				if( child.name == searchName ) {
					return child;
				} else {
					Transform childSearch = getChildRoot(child,searchName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
			
		protected void ChangeDoorType( string door ) {
			if( DoorsRoot != null ) {
							
				int loopCount = 0;
				foreach( Transform child in DoorsRoot )
				{
					child.gameObject.SetActive(false);
					if( loopCount == DoorTypeChoices.IndexOf(door) ) {
						child.gameObject.SetActive(true);
					}
					
					loopCount++;
				}
			}
		}
		
		protected void ChangeLedsState( bool lit ) {
			if( LedsRoot != null ) {
							
				int loopCount = 0;
				foreach( Transform child in LedsRoot )
				{
					child.gameObject.SetActive(false);
					if((child.name == "leds_on" && lit == true) || (child.name == "leds_off" && lit == false)) {
						child.gameObject.SetActive(true);
					}
					
					loopCount++;
				}
			}
		}
		
		protected void ChangeMirrorLedState( bool lit ) {
			if( MirrorLedRoot != null ) {
							
				int loopCount = 0;
				foreach( Transform child in MirrorLedRoot )
				{
					child.gameObject.SetActive(false);
					if((child.name == "led_on" && lit == true) || (child.name == "led_off" && lit == false)) {
						child.gameObject.SetActive(true);
					}
					
					loopCount++;
				}
			}
		}
		
		protected void ChangeTransparencyState( bool transparent ) {
			if( TransparencyRoot != null ) {
							
				int loopCount = 0;
				foreach( Transform child in TransparencyRoot )
				{
					Renderer childRenderer = child.GetComponent<Renderer>();
					if (childRenderer)
					{
						childRenderer.enabled = !transparent;
					}
				}
			}
		}
		
		
		// **************************
		// Callbacks & similar
		// **************************
		protected void DoorTypeCallback(string door) {
			ChangeDoorType( door );
		}

		protected void EnableShowerLedsCallback(bool enabled) {
			if (LedsRoot != null)
			{
				ChangeLedsState(enabled);
			}
		}
		
		protected void EnableMirrorLedCallback(bool enabled) {
			if (MirrorLedRoot != null)
			{
				ChangeMirrorLedState(enabled);
			}
		}
		
		protected void EnableHideWallsCallback(bool enabled) {
			if (TransparencyRoot != null)
			{
				ChangeTransparencyState(enabled);
			}
		}	
			
		// **************************
		// COROUTINES
		// **************************
		
		// Routine checking every 5secs if our CUA is still valid, this will invalidate the UI if the user change the CUA in the asset tab
		private IEnumerator CheckCUAStatus()
		{
			while (cuaIsValid)
			{
				// Our object root is null, it means our CUA is invalid
				if (ObjectRoot == null)
				{
					// We invalidate it so that the coroutine stops by itself, remove the UI and show the UI to alert the user
					cuaIsValid = false;
					ClearCUAUI();
					CreateCUAInvalidatedUI();
				}
				yield return new WaitForSeconds(5.0f);
			}
		}

		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}


	}
}